import { MapDrawActions } from '@/components/Map/MapDrawActions/MapDrawActions.component';
import { fireEvent, render, screen } from '@testing-library/react';
import { MAP_EDIT_ACTIONS } from '@/redux/mapEditTools/mapEditTools.constants';
import { mapEditToolsSetActiveAction } from '@/redux/mapEditTools/mapEditTools.slice';
import {
  getReduxWrapperWithStore,
  InitialStoreState,
} from '@/utils/testing/getReduxWrapperWithStore';
import { StoreType } from '@/redux/store';
import { MAP_EDIT_TOOLS_STATE_INITIAL_MOCK } from '@/redux/mapEditTools/mapEditTools.mock';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import {
  LAYER_STATE_DEFAULT_DATA,
  LAYERS_STATE_INITIAL_LAYER_MOCK,
} from '@/redux/layers/layers.mock';
import { MAIN_MAP_ID } from '@/constants/mocks';
import { layerFixture } from '@/models/fixtures/layerFixture';

jest.mock('../../../redux/hooks/useAppDispatch', () => ({
  useAppDispatch: jest.fn(),
}));

const renderComponent = (initialStoreState: InitialStoreState = {}): { store: StoreType } => {
  const { Wrapper, store } = getReduxWrapperWithStore(initialStoreState);

  return (
    render(
      <Wrapper>
        <MapDrawActions />
      </Wrapper>,
    ),
    {
      store,
    }
  );
};

describe('MapDrawActions', () => {
  const mockDispatch = jest.fn(() => {});

  beforeEach(() => {
    (useAppDispatch as jest.Mock).mockReturnValue(mockDispatch);
  });

  it('renders the MapDrawActionsButton and toggles action on click', () => {
    const layerId = 0;
    renderComponent({
      mapEditTools: MAP_EDIT_TOOLS_STATE_INITIAL_MOCK,
      layers: {
        [layerId]: {
          ...LAYERS_STATE_INITIAL_LAYER_MOCK,
          data: {
            ...LAYER_STATE_DEFAULT_DATA,
            layersVisibility: { [MAIN_MAP_ID]: true },
            layers: [
              {
                details: { ...layerFixture, id: MAIN_MAP_ID },
                texts: {},
                rects: [],
                ovals: [],
                lines: [],
                images: {},
              },
            ],
          },
        },
      },
    });
    const button = screen.getByRole('button', { name: /draw image/i });
    expect(button).toBeInTheDocument();
    fireEvent.click(button);

    expect(mockDispatch).toHaveBeenCalledWith(
      mapEditToolsSetActiveAction(MAP_EDIT_ACTIONS.DRAW_IMAGE),
    );
  });
});
