/* eslint-disable no-magic-numbers */
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { ModelElementsState } from '@/redux/modelElements/modelElements.types';
import modelElementsReducer from '@/redux/modelElements/modelElements.slice';
import { getModelElementsForModel } from '@/redux/modelElements/modelElements.thunks';
import { modelElementsFixture } from '@/models/fixtures/modelElementsFixture';

const mockedAxiosClient = mockNetworkNewAPIResponse();

describe('model elements thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<ModelElementsState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('modelElements', modelElementsReducer);
  });

  describe('getModelElementsForModel', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient
        .onGet(apiPath.getModelElements(0))
        .reply(HttpStatusCode.Ok, modelElementsFixture);

      const { payload } = await store.dispatch(getModelElementsForModel(0));
      expect(payload).toEqual(modelElementsFixture.content);
    });

    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getModelElements(0))
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getModelElementsForModel(0));
      expect(payload).toEqual(undefined);
    });
  });
});
